/*
Byte/bit exchanger for converting Capture cart raw dumps
Based upon original code by Marko Mkel <Marko.Makela@HUT.FI>
Capture adaption by streetuff/trsi <streetuff@trsi.de>

Capture Cart layout according to photos by D.Mackey:

Chip pin --- Cartridge port
A0 --------- Y/A0
A1 --------- X/A1
A2 --------- V/A2
A3 --------- U/A10
A4 --------- S/A3
A5 --------- P/A4
A6 --------- M/A11
A7 --------- L/A5
A8 --------- N/A9
A9 --------- R/A6
A10 -------- W/A8
A11 -------- T/A7
A12 -------- J/A12

D0 --------- 17/D3
D1 --------- 18/D2
D2 --------- 20/D4
D3 --------- 21/D1
D4 --------- 19/D5
D5 --------- 16/D0
D6 --------- 15/D6
D7 --------- 14/D7

*/
#include <stdio.h>

static unsigned char databus[256];
static unsigned char image[1 << 13];

#define MVBIT(i,from,to) (((i) & 1 << (from)) ? 1 << (to) : 0)

int main (int argc, char** argv)
{
  unsigned i;
  FILE *hin,*hout;
  if(argc<3)
    return 1;
  hin =fopen(argv[1],"rb");
  if(!hin)
    return 2;
  hout=fopen(argv[2],"wb");
  if(!hout)
    return 3;

  for (i = 255; i; i--)
    databus[i] =
      MVBIT (i, 3, 0) |
      MVBIT (i, 2, 1) |
      MVBIT (i, 4, 2) |
      MVBIT (i, 1, 3) |
      MVBIT (i, 5, 4) |
      MVBIT (i, 0, 5) |
      MVBIT (i, 6, 6) |
      MVBIT (i, 7, 7);

  for (i = 0; i < sizeof image; i++) {
    int c = getc(hin);
    if (c == EOF)
      break;
    else
      image[MVBIT (i, 0, 0) |
            MVBIT (i, 1, 1) |
            MVBIT (i, 2, 2) |
            MVBIT (i, 10, 3) |
            MVBIT (i, 3, 4) |
            MVBIT (i, 4, 5) |
            MVBIT (i, 11, 6) |
            MVBIT (i, 5, 7) |
            MVBIT (i, 9, 8) |
            MVBIT (i, 6, 9) |
            MVBIT (i, 8, 10) |
            MVBIT (i, 7, 11) |
            MVBIT (i, 12, 12) ] = databus[(unsigned char) c];
  }

  return fwrite (image, 1, sizeof image, hout);
}
